/*
    This file is part of KAddressBook.                                  
    Copyright (c) 2002 Mike Pilone <mpilone@slac.com>                   
                                                                        
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or   
    (at your option) any later version.                                 
                                                                        
    This program is distributed in the hope that it will be useful,     
    but WITHOUT ANY WARRANTY; without even the implied warranty of      
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the        
    GNU General Public License for more details.                        
                                                                        
    You should have received a copy of the GNU General Public License   
    along with this program; if not, write to the Free Software         
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           
                                                                        
    As a special exception, permission is given to link this program    
    with any edition of Qt, and distribute the resulting executable,    
    without including the source code for Qt in the source distribution.
*/                                                                      


#include "kaddressbookview.h"


#include <qlayout.h>
#include <qapplication.h>
#include <kdebug.h>
#include <klocale.h>
#include <kconfig.h>

#include "contactentrylist.h"

///////////////////////////////
// KAddressBookView

KAddressBookView::KAddressBookView(ContactEntryList *doc, QWidget *parent, 
                                   const char *name)
    : QWidget(parent, name), mDocument(doc), mFieldList()
{
    initGUI();
}

KAddressBookView::~KAddressBookView()
{
  kdDebug() << "KAddressBookView::~KAddressBookView: destroying - "
            << name() << endl;
}

void KAddressBookView::readConfig(KConfig *config)
{
  // I don't like the view reading its attribs directly. I would prefer
  // if they were passed in as a param, so the view manager can determine
  // defaults. Also, a view shouldn't save the attribs, so in my opinion it
  // shouldn't load them. -mpilone

  mFieldList = KABC::Field::restoreFields( config, "KABCFields" );

  if ( mFieldList.isEmpty() ) {
    mFieldList = KABC::Field::defaultFields();
  }
}

void KAddressBookView::writeConfig(KConfig *)
{
}

QString KAddressBookView::selectedEmails()
{
  bool first = true;
  QString emailAddrs;
  QStringList uidList = selectedUids();
  ContactEntry *ce;
  QString email;
  
  // Loop through the list of selected addressees, geting each one from the
  // document and asking it for the email address.
  QStringList::Iterator iter;
  for (iter = uidList.begin(); iter != uidList.end(); ++iter)
  {
      ce = mDocument->find(*iter);
      
      if ((ce != 0L) && (ce->find("EMAIL")))
      {
          email = *ce->find("EMAIL");
          if (!email.isEmpty())
          {
              email.stripWhiteSpace();

              QString sFileAs;
              if (ce->find( "N" )) 
              {
                  sFileAs = *ce->find( "N" );
                  sFileAs.stripWhiteSpace();
                  sFileAs += " ";
              }

              if (!first)
                  emailAddrs += ", ";
              else
                  first = false;

              // Append to the long string
              emailAddrs += sFileAs + "<" + email + ">";
          }
      }
  }
  
  return emailAddrs;
}

QStringList KAddressBookView::addresseeUids()
{
    // For now just return all the uids, since filtering hasn't been 
    // implemented
    return mDocument->keys();
}

void KAddressBookView::initGUI()
{
    // Create the layou
    QVBoxLayout *layout = new QVBoxLayout(this);
    
    // Add the view widget
    mViewWidget = new QWidget(this, "mViewWidget");
    layout->addWidget(mViewWidget);
    
}

void KAddressBookView::incrementalSearch(const QString &, 
                                         KABC::Field *)
{
    // Does nothing unless overloaded
}

#include "kaddressbookview.moc"
